@@ -6,7 +6,6 @@ import time
 import json
 
 from xml.dom import minidom
-from xml.parsers.expat import ExpatError
 
 from .messages import MESSAGE_TYPES, UnknownMessage
 from .exceptions import ParseError, NeedParseError, NeedParamError, OfficialAPIError
@@ -53,6 +52,9 @@ class WechatBasic(object):
         """
         self._check_token()
 
+        if not signature or not timestamp or not nonce:
+            return False
+
         tmp_list = [self.__token, timestamp, nonce]
         tmp_list.sort()
         tmp_str = ''.join(tmp_list)
@@ -77,7 +79,7 @@ class WechatBasic(object):
 
         try:
             doc = minidom.parseString(data)
-        except ExpatError:
+        except Exception:
             raise ParseError()
 
         params = [ele for ele in doc.childNodes[0].childNodes
@@ -122,6 +124,7 @@ class WechatBasic(object):
         :return: 符合微信服务器要求的 XML 响应数据
         """
         self._check_parse()
+        content = self._transcoding(content)
 
         return TextReply(message=self.__message, content=content).render()
 
@@ -154,6 +157,8 @@ class WechatBasic(object):
         :return: 符合微信服务器要求的 XML 响应数据
         """
         self._check_parse()
+        title = self._transcoding(title)
+        description = self._transcoding(description)
 
         return VideoReply(message=self.__message, media_id=media_id, title=title, description=description).render()
 
@@ -168,6 +173,10 @@ class WechatBasic(object):
         :return: 符合微信服务器要求的 XML 响应数据
         """
         self._check_parse()
+        music_url = self._transcoding(music_url)
+        title = self._transcoding(title)
+        description = self._transcoding(description)
+        hq_music_url = self._transcoding(hq_music_url)
 
         return MusicReply(message=self.__message, title=title, description=description, music_url=music_url,
                           hq_music_url=hq_music_url, thumb_media_id=thumb_media_id).render()
@@ -179,6 +188,15 @@ class WechatBasic(object):
         :return: 符合微信服务器要求的 XML 响应数据
         """
         self._check_parse()
+        for article in articles:
+            if article.get('title'):
+                article['title'] = self._transcoding(article['title'])
+            if article.get('description'):
+                article['description'] = self._transcoding(article['description'])
+            if article.get('picurl'):
+                article['picurl'] = self._transcoding(article['picurl'])
+            if article.get('url'):
+                article['url'] = self._transcoding(article['url'])
 
         news = ArticleReply(message=self.__message)
         for article in articles:
@@ -189,7 +207,7 @@ class WechatBasic(object):
     def grant_token(self):
         """
         获取 Access Token
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=通用接口文档
+        详情请参考 http://mp.weixin.qq.com/wiki/11/0e4b294685f817b95cbed85ba5e82b8f.html
         :return: 返回的 JSON 数据包
         """
         self._check_appid_appsecret()
@@ -212,37 +230,37 @@ class WechatBasic(object):
                 'button':[
                     {
                         'type':'click',
-                        'name':'今日歌曲',
+                        'name':u'今日歌曲',
                         'key':'V1001_TODAY_MUSIC'
                     },
                     {
                         'type':'click',
-                        'name':'歌手简介',
+                        'name':u'歌手简介',
                         'key':'V1001_TODAY_SINGER'
                     },
                     {
-                        'name':'菜单',
+                        'name':u'菜单',
                         'sub_button':[
                             {
                                 'type':'view',
-                                'name':'搜索',
+                                'name':u'搜索',
                                 'url':'http://www.soso.com/'
                             },
                             {
                                 'type':'view',
-                                'name':'视频',
+                                'name':u'视频',
                                 'url':'http://v.qq.com/'
                             },
                             {
                                 'type':'click',
-                                'name':'赞一下我们',
+                                'name':u'赞一下我们',
                                 'key':'V1001_GOOD'
                             }
                         ]
                     }
                 ]})
 
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=自定义菜单创建接口
+        详情请参考 http://mp.weixin.qq.com/wiki/13/43de8269be54a0a6f64413e4dfa94f39.html
         :param menu_data: Python 字典
         :return: 返回的 JSON 数据包
         """
@@ -256,7 +274,7 @@ class WechatBasic(object):
     def get_menu(self):
         """
         查询自定义菜单
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=自定义菜单查询接口
+        详情请参考 http://mp.weixin.qq.com/wiki/16/ff9b7b85220e1396ffa16794a9d95adc.html
         :return: 返回的 JSON 数据包
         """
         self._check_appid_appsecret()
@@ -266,7 +284,7 @@ class WechatBasic(object):
     def delete_menu(self):
         """
         删除自定义菜单
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=自定义菜单删除接口
+        详情请参考 http://mp.weixin.qq.com/wiki/16/8ed41ba931e4845844ad6d1eeb8060c8.html
         :return: 返回的 JSON 数据包
         """
         self._check_appid_appsecret()
@@ -276,7 +294,7 @@ class WechatBasic(object):
     def upload_media(self, media_type, media_file):
         """
         上传多媒体文件
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=上传下载多媒体文件
+        详情请参考 http://mp.weixin.qq.com/wiki/10/78b15308b053286e2a66b33f0f0f5fb6.html
         :param media_type: 媒体文件类型，分别有图片（image）、语音（voice）、视频（video）和缩略图（thumb）
         :param media_file:要上传的文件，一个 File-object
         :return: 返回的 JSON 数据包
@@ -297,7 +315,7 @@ class WechatBasic(object):
     def download_media(self, media_id):
         """
         下载多媒体文件
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=上传下载多媒体文件
+        详情请参考 http://mp.weixin.qq.com/wiki/10/78b15308b053286e2a66b33f0f0f5fb6.html
         :param media_id: 媒体文件 ID
         :return: requests 的 Response 实例
         """
@@ -315,7 +333,7 @@ class WechatBasic(object):
     def create_group(self, name):
         """
         创建分组
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=分组管理接口
+        详情请参考 http://mp.weixin.qq.com/wiki/13/be5272dc4930300ba561d927aead2569.html
         :param name: 分组名字（30个字符以内）
         :return: 返回的 JSON 数据包
         """
@@ -333,7 +351,7 @@ class WechatBasic(object):
     def get_groups(self):
         """
         查询所有分组
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=分组管理接口
+        详情请参考 http://mp.weixin.qq.com/wiki/13/be5272dc4930300ba561d927aead2569.html
         :return: 返回的 JSON 数据包
         """
         self._check_appid_appsecret()
@@ -343,7 +361,7 @@ class WechatBasic(object):
     def get_group_by_id(self, openid):
         """
         查询用户所在分组
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=分组管理接口
+        详情请参考 http://mp.weixin.qq.com/wiki/13/be5272dc4930300ba561d927aead2569.html
         :param openid: 用户的OpenID
         :return: 返回的 JSON 数据包
         """
@@ -359,7 +377,7 @@ class WechatBasic(object):
     def update_group(self, group_id, name):
         """
         修改分组名
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=分组管理接口
+        详情请参考 http://mp.weixin.qq.com/wiki/13/be5272dc4930300ba561d927aead2569.html
         :param group_id: 分组id，由微信分配
         :param name: 分组名字（30个字符以内）
         :return: 返回的 JSON 数据包
@@ -379,7 +397,7 @@ class WechatBasic(object):
     def move_user(self, user_id, group_id):
         """
         移动用户分组
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=分组管理接口
+        详情请参考 http://mp.weixin.qq.com/wiki/13/be5272dc4930300ba561d927aead2569.html
         :param user_id: 用户 ID 。 就是你收到的 WechatMessage 的 source
         :param group_id: 分组 ID
         :return: 返回的 JSON 数据包
@@ -397,7 +415,7 @@ class WechatBasic(object):
     def get_user_info(self, user_id, lang='zh_CN'):
         """
         获取用户基本信息
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=获取用户基本信息
+        详情请参考 http://mp.weixin.qq.com/wiki/14/bb5031008f1494a59c6f71fa0f319c66.html
         :param user_id: 用户 ID, 就是你收到的 WechatMessage 的 source
         :param lang: 返回国家地区语言版本，zh_CN 简体，zh_TW 繁体，en 英语
         :return: 返回的 JSON 数据包
@@ -416,7 +434,7 @@ class WechatBasic(object):
     def get_followers(self, first_user_id=None):
         """
         获取关注者列表
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=获取关注者列表
+        详情请参考 http://mp.weixin.qq.com/wiki/3/17e6919a39c1c53555185907acf70093.html
         :param first_user_id: 可选。第一个拉取的OPENID，不填默认从头开始拉取
         :return: 返回的 JSON 数据包
         """
@@ -432,7 +450,7 @@ class WechatBasic(object):
     def send_text_message(self, user_id, content):
         """
         发送文本消息
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=发送客服消息
+        详情请参考 http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html
         :param user_id: 用户 ID, 就是你收到的 WechatMessage 的 source
         :param content: 消息正文
         :return: 返回的 JSON 数据包
@@ -453,7 +471,7 @@ class WechatBasic(object):
     def send_image_message(self, user_id, media_id):
         """
         发送图片消息
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=发送客服消息
+        详情请参考 http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html
         :param user_id: 用户 ID, 就是你收到的 WechatMessage 的 source
         :param media_id: 图片的媒体ID。 可以通过 :func:`upload_media` 上传。
         :return: 返回的 JSON 数据包
@@ -474,7 +492,7 @@ class WechatBasic(object):
     def send_voice_message(self, user_id, media_id):
         """
         发送语音消息
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=发送客服消息
+        详情请参考 http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html
         :param user_id: 用户 ID, 就是你收到的 WechatMessage 的 source
         :param media_id: 发送的语音的媒体ID。 可以通过 :func:`upload_media` 上传。
         :return: 返回的 JSON 数据包
@@ -495,7 +513,7 @@ class WechatBasic(object):
     def send_video_message(self, user_id, media_id, title=None, description=None):
         """
         发送视频消息
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=发送客服消息
+        详情请参考 http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html
         :param user_id: 用户 ID, 就是你收到的 WechatMessage 的 source
         :param media_id: 发送的视频的媒体ID。 可以通过 :func:`upload_media` 上传。
         :param title: 视频消息的标题
@@ -524,7 +542,7 @@ class WechatBasic(object):
     def send_music_message(self, user_id, url, hq_url, thumb_media_id, title=None, description=None):
         """
         发送音乐消息
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=发送客服消息
+        详情请参考 http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html
         :param user_id: 用户 ID, 就是你收到的 WechatMessage 的 source
         :param url: 音乐链接
         :param hq_url: 高品质音乐链接，wifi环境优先使用该链接播放音乐
@@ -557,7 +575,7 @@ class WechatBasic(object):
     def send_article_message(self, user_id, articles):
         """
         发送图文消息
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=发送客服消息
+        详情请参考 http://mp.weixin.qq.com/wiki/7/12a5a320ae96fecdf0e15cb06123de9f.html
         :param user_id: 用户 ID, 就是你收到的 WechatMessage 的 source
         :param articles: list 对象, 每个元素为一个 dict 对象, key 包含 `title`, `description`, `picurl`, `url`
         :return: 返回的 JSON 数据包
@@ -587,7 +605,7 @@ class WechatBasic(object):
     def create_qrcode(self, **data):
         """
         创建二维码
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=生成带参数的二维码
+        详情请参考 http://mp.weixin.qq.com/wiki/18/28fc21e7ed87bec960651f0ce873ef8a.html
         :param data: 你要发送的参数 dict
         :return: 返回的 JSON 数据包
         """
@@ -601,7 +619,7 @@ class WechatBasic(object):
     def show_qrcode(self, ticket):
         """
         通过ticket换取二维码
-        详情请参考 http://mp.weixin.qq.com/wiki/index.php?title=生成带参数的二维码
+        详情请参考 http://mp.weixin.qq.com/wiki/18/28fc21e7ed87bec960651f0ce873ef8a.html
         :param ticket: 二维码 ticket 。可以通过 :func:`create_qrcode` 获取到
         :return: 返回的 Request 对象
         """
@@ -710,4 +728,22 @@ class WechatBasic(object):
             method="post",
             url=url,
             **kwargs
-        )
\ No newline at end of file
+        )
+
+    def _transcoding(self, data):
+        """
+        编码转换
+        :param data: 需要转换的数据
+        :return: 转换好的数据
+        """
+        if not data:
+            return data
+
+        result = None
+        if type(data) == unicode:
+            result = data
+        elif type(data) == str:
+            result = data.decode('utf-8')
+        else:
+            raise ParseError()
+        return result
\ No newline at end of file
