@@ -47,4 +47,13 @@ class LoginError(UnOfficialAPIError):
     """
     微信非官方API请求出错异常 - 登录出错
     """
-    pass
\ No newline at end of file
+    pass
+
+
+class LoginVerifyCodeError(LoginError):
+    """
+    微信非官方API请求出错异常 - 登录出错 - 验证码错误
+    """
+    pass
+
+
