@@ -6,7 +6,7 @@ import requests
 import json
 import random
 
-from .exceptions import UnOfficialAPIError, NeedLoginError, LoginError
+from .exceptions import UnOfficialAPIError, NeedLoginError, LoginError, LoginVerifyCodeError
 
 
 class WechatExt(object):
@@ -15,13 +15,14 @@ class WechatExt(object):
 
     通过模拟登陆的方式实现更多的高级功能, 请注意使用本类有风险, 请自行承担
     """
-    def __init__(self, username, password, token=None, cookies=None, ifencodepwd=False):
+    def __init__(self, username, password, token=None, cookies=None, ifencodepwd=False, login=True):
         """
         :param username: 你的微信公众平台账户用户名
         :param password: 你的微信公众平台账户密码
         :param token: 直接导入的 ``token`` 值, 该值需要在上一次该类实例化之后手动进行缓存并在此传入, 如果不传入, 将会在实例化的时候自动获取
         :param cookies: 直接导入的 ``cookies`` 值, 该值需要在上一次该类实例化之后手动进行缓存并在此传入, 如果不传入, 将会在实例化的时候自动获取
         :param ifencodepwd: 密码是否已经经过编码, 如果密码已经经过加密, 此处为 ``True`` , 如果传入的密码为明文, 此处为 ``False``
+        :param login: 是否在初始化过程中尝试登录 (推荐此处设置为 ``False``, 然后手动执行登录以方便进行识别验证码等操作, 此处默认值为 ``True`` 为兼容历史版本
         :return:
         """
         self.__username = username
@@ -32,43 +33,80 @@ class WechatExt(object):
         self.__cookies = cookies
         self.__lastmsgid = 0
         self.__token = token
+
         self.__ticket = None
         self.__ticket_id = None
+        self.__fakeid = None
 
         if not self.__token or not self.__cookies:
             self.__token = ''
             self.__cookies = ''
-            self.login()
+            if login:
+                self.login()
 
-    def login(self):
+    def login(self, verify_code=''):
         """
         登录微信公众平台
         注意在实例化 ``WechatExt`` 的时候，如果没有传入 ``token`` 及 ``cookies`` ，将会自动调用该方法，无需手动调用
         当且仅当捕获到 ``NeedLoginError`` 异常时才需要调用此方法进行登录重试
+        :param verify_code: 验证码, 不传入则为无验证码
+        :raises LoginVerifyCodeError: 需要验证码或验证码出错，该异常为 ``LoginError`` 的子类
         :raises LoginError: 登录出错异常，异常内容为微信服务器响应的内容，可作为日志记录下来
         """
-        url = 'https://mp.weixin.qq.com/cgi-bin/login?lang=zh_CN'
+        url = 'https://mp.weixin.qq.com/cgi-bin/login'
         payload = {
             'username': self.__username,
-            'imgcode': '',
-            'f': 'json',
             'pwd': self.__password,
+            'imgcode': verify_code,
+            'f': 'json',
         }
         headers = {
             'x-requested-with': 'XMLHttpRequest',
             'referer': 'https://mp.weixin.qq.com/cgi-bin/loginpage?t=wxm2-login&lang=zh_CN',
+            'Cookie': self.__cookies,
         }
         r = requests.post(url, data=payload, headers=headers)
 
         s = re.search(r'token=(\d+)', r.text)
         if not s:
-            raise LoginError(r.text)
+            try:
+                error_code = json.loads(r.text)['base_resp']['ret']
+            except (KeyError, ValueError):
+                raise LoginError(r.text)
+
+            if error_code in [-8, -27]:
+                raise LoginVerifyCodeError(r.text)
+            else:
+                raise LoginError(r.text)
         self.__token = int(s.group(1))
 
         self.__cookies = ''
         for cookie in r.cookies:
             self.__cookies += cookie.name + '=' + cookie.value + ';'
 
+    def get_verify_code(self, file_path):
+        """
+        获取登录验证码并存储
+        :param file_path: 将验证码图片保存的文件路径
+        """
+        url = 'https://mp.weixin.qq.com/cgi-bin/verifycode'
+        payload = {
+            'username': self.__username,
+            'r': int(random.random() * 10000000000000),
+        }
+        headers = {
+            'referer': 'https://mp.weixin.qq.com/',
+        }
+        r = requests.get(url, data=payload, headers=headers, stream=True)
+
+        self.__cookies = ''
+        for cookie in r.cookies:
+            self.__cookies += cookie.name + '=' + cookie.value + ';'
+
+        with open(file_path, 'wb') as fd:
+            for chunk in r.iter_content(1024):
+                fd.write(chunk)
+
     def get_token_cookies(self):
         """
         获取当前 token 及 cookies, 供手动缓存使用
@@ -346,9 +384,20 @@ class WechatExt(object):
 
         return message
 
-    def get_dialog_message(self, fakeid):
+    def get_dialog_message(self, fakeid, last_msgid=0, create_time=0):
         """
-        获取与指定用户的对话内容
+        获取与指定用户的对话内容, 获取的内容由 ``last_msgid`` (需要获取的对话中时间最早的 **公众号发送给用户** 的消息ID) 和 ``create_time`` (需要获取的对话中时间最早的消息时间戳) 进行过滤
+
+        消息过滤规则:
+
+        1. 首先按照 ``last_msgid`` 过滤 (不需要按照 ``last_msgid`` 过滤则不需要传入此参数)
+
+            a. ``fakeid`` 为用户 UID
+            b. 通过 ``last_msgid`` 去匹配公众号过去发送给用户的某一条消息
+            c. 如果匹配成功, 则返回这条消息之后与这个用户相关的所有消息内容 (包括发送的消息和接收的)
+            d. 如果匹配失败 (没有找到), 则返回与这个用户相关的所有消息 (包括发送的消息和接收的)
+
+        2. 第一条规则返回的消息内容接着按照 ``create_time`` 进行过滤, 返回 ``create_time`` 时间戳之时及之后的所有消息 (不需要按照 ``create_time`` 过滤则不需要传入此参数)
 
         返回JSON示例::
 
@@ -418,24 +467,29 @@ class WechatExt(object):
                 }
             }
 
-        :param fakeid: 用户 UID (即 fakeid)
+        :param fakeid: 用户 UID (即 fakeid )
+        :param last_msgid: 公众号之前发送给用户(fakeid)的消息 ID, 为 0 则表示全部消息
+        :param create_time: 获取这个时间戳之时及之后的消息，为 0 则表示全部消息
         :return: 返回的 JSON 数据
         :raises NeedLoginError: 操作未执行成功, 需要再次尝试登录, 异常内容为服务器返回的错误数据
         """
-        url = 'https://mp.weixin.qq.com/cgi-bin/singlesendpage?t=message/send&action=index&tofakeid={fakeid}&token={token}&lang=zh_CN&f=json&random={random}'.format(
+        self._init_fakeid()
+
+        url = 'https://mp.weixin.qq.com/cgi-bin/singlesendpage?tofakeid={fakeid}&action=sync&lastmsgfromfakeid={fromfakeid}&lastmsgid={last_msgid}&createtime={create_time}&token={token}&lang=zh_CN&f=json&ajax=1'.format(
             fakeid=fakeid,
+            fromfakeid=self.__fakeid,
+            last_msgid=last_msgid,
+            create_time=create_time,
             token=self.__token,
-            random=round(random.random(), 3),
         )
+
         headers = {
             'x-requested-with': 'XMLHttpRequest',
-            'referer': 'https://mp.weixin.qq.com/cgi-bin/masssendpage?t=mass/send&token={token}&lang=zh_CN'.format(
-                token=self.__token,
-            ),
+            'referer': 'https://mp.weixin.qq.com/cgi-bin/message?t=message/list&count=20&day=7&token={token}&lang=zh_CN'.format(token=self.__token),
             'cookie': self.__cookies,
         }
-        r = requests.get(url, headers=headers)
 
+        r = requests.get(url, headers=headers)
         try:
             message = json.dumps(json.loads(r.text)['page_info'], ensure_ascii=False)
         except (KeyError, ValueError):
@@ -453,13 +507,16 @@ class WechatExt(object):
         """
         url = 'https://mp.weixin.qq.com/cgi-bin/singlesend?t=ajax-response'
         payload = {
+            'lang': 'zh_CN',
+            'f': 'json',
             'tofakeid': fakeid,
             'type': 10,
             'token': self.__token,
-            'fid': msgid,
             'appmsgid': msgid,
+            'app_id': msgid,
             'error': 'false',
             'ajax': 1,
+            'random': random.random(),
         }
         headers = {
             'x-requested-with': 'XMLHttpRequest',
@@ -484,6 +541,81 @@ class WechatExt(object):
         except KeyError:
             raise NeedLoginError(r.text)
 
+    def add_news(self, news):
+        """
+        在素材库中创建图文消息
+
+        :param news: list 对象, 其中的每个元素为一个 dict 对象, 代表一条图文, key 值分别为 ``title``, ``author``, ``summary``,
+                     ``content``, ``picture_id``, ``from_url``, 对应内容为标题, 作者, 摘要, 内容, 素材库里的
+                     图片ID(可通过 ``upload_file`` 函数上传获取), 来源链接。
+
+                     其中必须提供的 key 值为 ``title`` 和 ``content``
+
+                     示例::
+
+                         [
+                             {
+                                 'title': '图文标题',
+                                 'author': '图文作者',
+                                 'summary': '图文摘要',
+                                 'content': '图文内容',
+                                 'picture_id': '23412341',
+                                 'from_url': 'http://www.baidu.com',
+                             },
+                             {
+                                 'title': '最少图文标题',
+                                 'content': '图文内容',
+                             }
+                         ]
+        :raises ValueError: 参数提供错误时抛出
+        :raises NeedLoginError: 操作未执行成功, 需要再次尝试登录, 异常内容为服务器返回的错误数据
+        """
+        if not news:
+            raise ValueError('The news cannot be empty')
+        for item in news:
+            if 'title' not in item or 'content' not in item:
+                raise ValueError('The news item needs to provide at least two arguments: title, content')
+
+        url = 'https://mp.weixin.qq.com/cgi-bin/operate_appmsg?lang=zh_CN&t=ajax-response&sub=create&token={token}'.format(
+            token=self.__token,
+        )
+        payload = {
+            'token': self.__token,
+            'type': 10,
+            'lang': 'zh_CN',
+            'sub': 'create',
+            'ajax': 1,
+            'AppMsgId': '',
+            'error': 'false',
+        }
+        headers = {
+            'referer': 'https://mp.weixin.qq.com/cgi-bin/operate_appmsg?lang=zh_CN&sub=edit&t=wxm-appmsgs-edit-new&type=10&subtype=3&token={token}'.format(
+                token=self.__token
+            ),
+            'cookie': self.__cookies,
+        }
+        i = 0
+        for item in news:
+            payload['title'+str(i)] = item.get('title')
+            payload['author'+str(i)] = item.get('author')
+            payload['digest'+str(i)] = item.get('summary')
+            payload['content'+str(i)] = item.get('content')
+            payload['fileid'+str(i)] = item.get('picture_id')
+            payload['sourceurl'+str(i)] = item.get('from_url')
+            i += 1
+        payload['count'] = i
+        r = requests.post(url, data=payload, headers=headers)
+
+        try:
+            message = json.loads(r.text)
+        except ValueError:
+            raise NeedLoginError(r.text)
+        try:
+            if message['ret'] != '0':
+                raise ValueError(r.text)
+        except KeyError:
+            raise NeedLoginError(r.text)
+
     def upload_file(self, filepath):
         """
         上传素材 (图片/音频/视频)
@@ -492,8 +624,7 @@ class WechatExt(object):
         :raises NeedLoginError: 操作未执行成功, 需要再次尝试登录, 异常内容为服务器返回的错误数据
         :raises ValueError: 参数出错, 错误原因直接打印异常即可 (常见错误内容: ``file not exist``: 找不到本地文件, ``audio too long``: 音频文件过长, ``file invalid type``: 文件格式不正确, 还有其他错误请自行检查)
         """
-        if not self.__ticket:
-            self._init_ticket()
+        self._init_ticket()
 
         url = 'https://mp.weixin.qq.com/cgi-bin/filetransfer?action=upload_material&f=json&ticket_id={ticket_id}&ticket={ticket}&token={token}&lang=zh_CN'.format(
             ticket_id=self.__ticket_id,
@@ -532,25 +663,43 @@ class WechatExt(object):
         向特定用户发送媒体文件
         :param fakeid: 用户 UID (即 fakeid)
         :param fid: 文件 ID
-        :param type: 文件类型 (2: 图片, 3: 音频, 4: 视频)
+        :param type: 文件类型 (2: 图片, 3: 音频, 15: 视频)
         :raises NeedLoginError: 操作未执行成功, 需要再次尝试登录, 异常内容为服务器返回的错误数据
         :raises ValueError: 参数出错, 错误原因直接打印异常即可 (常见错误内容: ``system error`` 或 ``can not send this type of msg``: 文件类型不匹配, ``user not exist``: 用户 fakeid 不存在, ``file not exist``: 文件 fid 不存在, 还有其他错误请自行检查)
         """
+        if type == 4:  # 此处判断为兼容历史版本, 微信官方已经将视频类型修改为 15
+            type = 15
+
         url = 'https://mp.weixin.qq.com/cgi-bin/singlesend?t=ajax-response&f=json&token={token}&lang=zh_CN'.format(
             token=self.__token,
         )
-        payloads = {
-            'token': self.__token,
-            'lang': 'zh_CN',
-            'f': 'json',
-            'ajax': 1,
-            'random': random.random(),
-            'type': type,
-            'file_id': fid,
-            'tofakeid': fakeid,
-            'fileid': fid,
-            'imgcode': '',
-        }
+        payloads = {}
+        if type == 2 or type == 3:  # 如果文件类型是图片或者音频
+            payloads = {
+                'token': self.__token,
+                'lang': 'zh_CN',
+                'f': 'json',
+                'ajax': 1,
+                'random': random.random(),
+                'type': type,
+                'file_id': fid,
+                'tofakeid': fakeid,
+                'fileid': fid,
+                'imgcode': '',
+            }
+        elif type == 15:  # 如果文件类型是视频
+            payloads = {
+                'token': self.__token,
+                'lang': 'zh_CN',
+                'f': 'json',
+                'ajax': 1,
+                'random': random.random(),
+                'type': type,
+                'app_id': fid,
+                'tofakeid': fakeid,
+                'appmsgid': fid,
+                'imgcode': '',
+            }
         headers = {
             'referer': 'https://mp.weixin.qq.com/cgi-bin/singlesendpage?tofakeid={fakeid}&t=message/send&action=index&token={token}&lang=zh_CN'.format(
                 fakeid=fakeid,
@@ -665,7 +814,7 @@ class WechatExt(object):
         :raises NeedLoginError: 操作未执行成功, 需要再次尝试登录, 异常内容为服务器返回的错误数据
         :raises ValueError: 参数出错, 错误原因直接打印异常即可 (常见错误内容: ``system error`` 或 ``can not send this type of msg``: 文件类型不匹配, ``user not exist``: 用户 fakeid 不存在, ``file not exist``: 文件 fid 不存在, 还有其他错误请自行检查)
         """
-        return self.send_file(fakeid, fid, 4)
+        return self.send_file(fakeid, fid, 15)
 
     def get_user_info(self, fakeid):
         """
@@ -982,9 +1131,25 @@ class WechatExt(object):
 
         return r.raw.data
 
+    def _init_fakeid(self):
+        """
+        初始化公众号自身的 ``fakeid`` 值
+        :raises NeedLoginError: 操作未执行成功, 需要再次尝试登录, 异常内容为服务器返回的错误数据
+        """
+        if not self.__fakeid:
+            self._init_self_information()
+
     def _init_ticket(self):
         """
-        初始化 Ticket 值
+        初始化公众号自身的 ``ticket`` 及 ``ticket_id`` 值
+        :raises NeedLoginError: 操作未执行成功, 需要再次尝试登录, 异常内容为服务器返回的错误数据
+        """
+        if not self.__ticket:
+            self._init_self_information()
+
+    def _init_self_information(self):
+        """
+        初始化公众号自身的属性值 (目前包括 ``Ticket`` 值 及 公众号自身的 ``fakeid`` 值)
         :raises NeedLoginError: 操作未执行成功, 需要再次尝试登录, 异常内容为服务器返回的错误数据
         """
         url = 'https://mp.weixin.qq.com/cgi-bin/home?t=home/index&lang=zh_CN&token={token}'.format(token=self.__token)
@@ -995,11 +1160,20 @@ class WechatExt(object):
         }
         r = requests.get(url, headers=headers)
 
+        # 获取 Ticket ID 值
         ticket_id = re.search(r'user_name:\"(.*)\"', r.text)
         if not ticket_id:
             raise NeedLoginError(r.text)
         self.__ticket_id = ticket_id.group(1)
+
+        # 获取 Ticket 值
         ticket = re.search(r'ticket:\"(.*)\"', r.text)
         if not ticket:
             raise NeedLoginError(r.text)
-        self.__ticket = ticket.group(1)
\ No newline at end of file
+        self.__ticket = ticket.group(1)
+
+        # 获取公众号自身的 fakeid 值
+        fakeid = re.search(r'uin:\"(.*)\"', r.text)
+        if not fakeid:
+            raise NeedLoginError(r.text)
+        self.__fakeid = fakeid.group(1)
\ No newline at end of file
